@@ -19,7 +19,7 @@
 #'   a file path, the output file path will have the same base name as the input
 #'   file, with an extension corresponding to the output format. The output
 #'   format is retrieved from the first value in the `output` field of the YAML
-#'   metadata of the `input` (e.g., `litedown::html_format` will generate HTML
+#'   metadata of the `input` (e.g., `html` will generate HTML
 #'   output). The `output` argument can also take an output format name
 #'   (possible values are `html`, `latex`, `xml`, `man`, `commonmark`, and
 #'   `text`). If no output format is detected or provided, the default is HTML.
@@ -98,9 +98,10 @@ mark = function(input, output = NULL, text = NULL, options = NULL, meta = list()
   # whether to write YAML metadata to output
   keep_yaml = isTRUE(options[['keep_yaml']])
 
-  # if keep_yaml, generate a fragment only, otherwise check the `template` value
-  # in the output format litedown::(html|latex)_format in YAML
-  template = if (keep_yaml) FALSE else yaml_field(yaml, format, 'template')
+  # if keep_yaml or format is not html/latex, don't use template; otherwise
+  # check the `template` value in litedown::(html|latex)_format in YAML
+  template = if (keep_yaml || !format %in% c('html', 'latex')) FALSE else
+    yaml_field(yaml, format, 'template')
   # if not set there, check global option; if not set, disable template if no
   # YAML was provided (i.e., generate a fragment)
   if (is.null(template))
@@ -134,12 +135,13 @@ mark = function(input, output = NULL, text = NULL, options = NULL, meta = list()
       # them (to avoid issues like #33) and restore them later
       text = one_string(text)
       text = match_replace(text, sprintf('`%s(?s).{3,}?%s`', id, id), function(x) {
-        n0 = length(maths)
-        maths <<- c(maths, gsub(sprintf('`%s|%s`', id, id), '', x))
         # replace math with !id-n-id! where n is the index of the math
-        sprintf('!%s-%d-%s!', id, n0 + seq_along(x), id)
+        tokens = sprintf('!%s-%d-%s!', id, length(maths) + seq_along(x), id)
+        math = gsub(sprintf('`%s|%s`', id, id), '', x)
+        maths <<- c(maths, set_names(math, tokens))
+        tokens
       })
-      if (format == 'html') maths = xfun::html_escape(maths)
+      if (format == 'html') maths = html_escape(maths)
       text = split_lines(text)
     }
   }
@@ -219,9 +221,11 @@ mark = function(input, output = NULL, text = NULL, options = NULL, meta = list()
       'LaTeX math expressions cannot be restored correctly (expected ',
       length(maths), ' expression(s) but found ', length(x), ' in the output).'
     )
-    maths
+    maths[x]
   })
 
+  has_mermaid = FALSE
+
   if (format == 'html') {
     # don't disable check boxes
     ret = gsub('(<li><input type="checkbox" [^>]*?)disabled="" (/>)', '\\1\\2', ret)
@@ -253,12 +257,8 @@ mark = function(input, output = NULL, text = NULL, options = NULL, meta = list()
     }, perl = FALSE)  # for perl = TRUE, we'd need (?s) before (.+?)
     # support mermaid
     r_mmd = '<pre><code class="language-mermaid">(.*?)</code></pre>'
-    if (length(grep(r_mmd, ret))) {
+    if (has_mermaid <- length(grep(r_mmd, ret))) {
       ret = gsub(r_mmd, '<pre class="mermaid">\\1</pre>', ret)
-      # add the js asset automatically if not detected
-      if (length(grep('mermaid', meta[['js']])) == 0) meta = add_meta(
-        meta, c(js = '@npm/mermaid/dist/mermaid.min.js')
-      )
     }
     r4 = '(<pre><code class="language-)\\{([^"]+)}">'
     # deal with ```{.class1 .class2 attrs}, which is not supported by commonmark
@@ -271,7 +271,9 @@ mark = function(input, output = NULL, text = NULL, options = NULL, meta = list()
       paste0(z1, z2, '>')
     }, 'html', function(z2) gsub(id4, ' ', restore_html(z2)))
     # some code blocks with "attributes" are verbatim ones
-    ret = match_replace(ret, '```+\\{.+}', function(x) gsub(id4, ' ', x, fixed = TRUE))
+    ret = match_replace(ret, '```+\\s*\\{.+}', function(x) gsub(id4, ' ', x, fixed = TRUE))
+    # remove empty table header
+    ret = gsub('<thead>\n<tr>\n(<th[^>]*></th>\n)+</tr>\n</thead>\n', '', ret)
     # table caption: a paragraph that starts with 'Table: ' or ': ' after </table>
     ret = gsub(
       '(<table>)(?s)(.+?</table>)\n<p>(Table)?: (?s)(.+?)</p>',
@@ -283,13 +285,13 @@ mark = function(input, output = NULL, text = NULL, options = NULL, meta = list()
     if (isTRUE(options[['number_sections']])) ret = number_sections(ret)
     # build table of contents
     ret = add_toc(ret, options)
-    # add js/css for math
-    if (!has_math) has_math = length(ret) &&
-      grepl('$$</p>', ret, fixed = TRUE)  # math may be from pkg_manual()'s HTML
+    # math
+    if (!has_math) has_math = length(ret) && (
+      grepl('$$</p>', ret, fixed = TRUE) || grepl('\\)</span>', ret, fixed = TRUE)
+    )  # math may be from pkg_manual()'s HTML
     is_katex = TRUE
     if (has_math && length(js_math <- js_options(options[['js_math']], 'katex'))) {
       is_katex = js_math$package == 'katex'
-      meta = set_math(meta, js_math, is_katex)
     }
     # number figures and tables, etc.
     ret = number_refs(ret, r_ref, is_katex)
@@ -330,7 +332,7 @@ mark = function(input, output = NULL, text = NULL, options = NULL, meta = list()
   }
 
   # convert some meta variables in case they use Markdown syntax
-  for (i in top_meta) if (meta_len <- length(meta[[i]])) {
+  if (is.character(template)) for (i in top_meta) if (meta_len <- length(meta[[i]])) {
     # if author is of length > 1, render them individually
     m_author = i == 'author' && meta_len > 1
     meta[[i]] = if (m_author) uapply(meta[[i]], render) else {
@@ -353,13 +355,23 @@ mark = function(input, output = NULL, text = NULL, options = NULL, meta = list()
   clever = isTRUE(options[['cleveref']])
   if (format == 'latex') ret = latex_refs(ret, r_ref, clever) else clever = FALSE
 
-  meta$body = ret
-
   # use the template (if provided) to create a standalone document
-  if (format %in% c('html', 'latex') && is.character(template)) {
-    # add HTML dependencies to `include-headers` if found
-    meta = add_html_deps(meta, output, 'local' %in% options[['embed_resources']])
-    ret = build_output(format, options, template, meta)
+  if (is.character(template)) {
+    meta$body = I(ret)
+    if (format == 'html') {
+      # reset the internal js/css stored in acc_var() on exit
+      on.exit(acc_var(), add = TRUE)
+      # add js/css for math
+      if (has_math) set_math(js_math, is_katex)
+      # add js/css for syntax highlighting
+      set_highlight(options, ret)
+      # add js for mermaid
+      if (has_mermaid && length(grep('mermaid', meta[['js']])) == 0)
+        acc_var(js = '@npm/mermaid/dist/mermaid.min.js')
+    }
+    ret = build_output(
+      format, options, template, meta, test = c(if (length(input)) dirname(input), '.')
+    )
     # load the cleveref package if not loaded
     if (clever && !any(grepl('\\\\usepackage.*\\{cleveref\\}', ret, perl = TRUE)))
       ret = sub('(?=\\\\begin\\{document\\})', '\\\\usepackage{cleveref}\n', ret, perl = TRUE)
@@ -394,44 +406,35 @@ mark = function(input, output = NULL, text = NULL, options = NULL, meta = list()
         )
       }
     }
-    # for RStudio to capture the output path when previewing the output (don't
-    # emit the message when rendering Rmd with rmarkdown::render() because
-    # render() will do it)
-    if (is_rmd_preview() && !'knit_meta' %in% ls(.env))
-      message('\nOutput created: ', output)
+    # for RStudio to capture the output path when previewing the output
+    if (is_rmd_preview()) message('\nOutput created: ', output)
     if (is_pdf) invisible(output) else write_utf8(ret, output)
   } else raw_string(ret)
 }
 
 # insert body and meta variables into a template
-build_output = function(format, options, template, meta) {
-  tpl = one_string(template, test = TRUE)
+build_output = function(format, options, template, meta, ...) {
+  tpl = one_string(template, ...)
   if (format == 'html') {
-    b = meta$body
-    set_meta = function(name, value) {
-      if (!name %in% names(meta)) meta[[name]] <<- value
-    }
-    set_meta('css', 'default')
-    set_meta('plain-title', I(str_trim(commonmark::markdown_text(meta[['title']]))))
-    meta = set_highlight(meta, options, b)
-    # if the class .line-numbers is present, add js/css for line numbers
-    if (any(grepl('<code class="[^"]*line-numbers', b))) for (i in c('css', 'js')) {
-      meta[[i]] = c(meta[[i]], '@code-line-numbers')
-    }
+    defaults = list(
+      'css' = 'default',
+      'plain-title' = I(str_trim(commonmark::markdown_text(meta[['title']])))
+    )
+    for (i in setdiff(names(defaults), names(meta))) meta[[i]] = defaults[[i]]
     # special handling for css/js "files" that have no extensions
-    for (i in c('css', 'js')) meta[[i]] = resolve_files(meta[[i]], i)
+    for (i in c('css', 'js')) meta[[i]] = resolve_files(c(meta[[i]], acc_var(i)), i)
   }
-  sub_vars(tpl, meta)
+  sub_vars(tpl, meta, ...)
 }
 
 # substitute all variables in template with their values
-sub_vars = function(tpl, meta) {
+sub_vars = function(tpl, meta, ...) {
   # find all variables in the template
   vars = unlist(match_full(tpl, '[$][-_[:alnum:]]+[$]'))
   # insert $body$ at last in case the body contain any $variables$ accidentally
   if (!is.na(i <- match('$body$', vars))) vars = c(vars[-i], vars[i])
   for (v in vars) {
-    tpl = sub_var(tpl, v, meta[[gsub('[$]', '', v)]])
+    tpl = sub_var(tpl, v, meta[[gsub('[$]', '', v)]], ...)
   }
   tpl
 }
